@@ -1,19 +1,10 @@
 // Copyright (C) 2020 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -63,7 +54,7 @@ impl ObjectSubclass for CustomSource {
 impl ObjectImpl for CustomSource {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
-            vec![glib::ParamSpec::new_object(
+            vec![glib::ParamSpecObject::new(
                 "source",
                 "Source",
                 "Source",
@@ -101,6 +92,8 @@ impl ObjectImpl for CustomSource {
     }
 }
 
+impl GstObjectImpl for CustomSource {}
+
 impl ElementImpl for CustomSource {
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: Lazy<Vec<gst::PadTemplate>> = Lazy::new(|| {
@@ -259,10 +252,8 @@ impl CustomSource {
         let mut stream_type = None;
 
         // Take stream type from stream-start event if we can
-        if let Some(event) = pad.sticky_event(gst::EventType::StreamStart, 0) {
-            if let gst::EventView::StreamStart(ev) = event.view() {
-                stream_type = ev.stream().map(|s| s.stream_type());
-            }
+        if let Some(ev) = pad.sticky_event::<gst::event::StreamStart<_>>(0) {
+            stream_type = ev.stream().map(|s| s.stream_type());
         }
 
         // Otherwise from the caps
