@@ -1,26 +1,16 @@
 // Copyright (C) 2020 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
 use gst::subclass::prelude::*;
 use gst::{gst_debug, gst_error, gst_info, gst_warning};
 
-use std::convert::TryFrom;
 use std::mem;
 use std::sync::Mutex;
 use std::time::Instant;
@@ -59,9 +49,9 @@ impl Default for Stats {
 impl Stats {
     fn to_structure(&self) -> gst::Structure {
         gst::Structure::builder("application/x-fallbacksrc-stats")
-            .field("num-retry", &self.num_retry)
-            .field("last-retry-reason", &self.last_retry_reason)
-            .field("buffering-percent", &self.buffering_percent)
+            .field("num-retry", self.num_retry)
+            .field("last-retry-reason", self.last_retry_reason)
+            .field("buffering-percent", self.buffering_percent)
             .build()
     }
 }
@@ -200,36 +190,36 @@ impl ObjectImpl for FallbackSrc {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "enable-audio",
                     "Enable Audio",
                     "Enable the audio stream, this will output silence if there's no audio in the configured URI",
                     true,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "enable-video",
                     "Enable Video",
                     "Enable the video stream, this will output black or the fallback video if there's no video in the configured URI",
                     true,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_string("uri", "URI", "URI to use", None, glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY),
-                glib::ParamSpec::new_object(
+                glib::ParamSpecString::new("uri", "URI", "URI to use", None, glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY),
+                glib::ParamSpecObject::new(
                     "source",
                     "Source",
                     "Source to use instead of the URI",
                     gst::Element::static_type(),
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_string(
+                glib::ParamSpecString::new(
                     "fallback-uri",
                     "Fallback URI",
                     "Fallback URI to use for video in case the main stream doesn't work",
                     None,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "timeout",
                     "Timeout",
                     "Timeout for switching to the fallback URI",
@@ -238,7 +228,7 @@ impl ObjectImpl for FallbackSrc {
                     5 * *gst::ClockTime::SECOND,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "restart-timeout",
                     "Timeout",
                     "Timeout for restarting an active source",
@@ -247,7 +237,7 @@ impl ObjectImpl for FallbackSrc {
                     5 * *gst::ClockTime::SECOND,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "retry-timeout",
                     "Retry Timeout",
                     "Timeout for stopping after repeated failure",
@@ -256,14 +246,14 @@ impl ObjectImpl for FallbackSrc {
                     60 * *gst::ClockTime::SECOND,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "restart-on-eos",
                     "Restart on EOS",
                     "Restart source on EOS",
                     false,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_enum(
+                glib::ParamSpecEnum::new(
                     "status",
                     "Status",
                     "Current source status",
@@ -271,7 +261,7 @@ impl ObjectImpl for FallbackSrc {
                     Status::Stopped as i32,
                     glib::ParamFlags::READABLE,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "min-latency",
                     "Minimum Latency",
                     "When the main source has a higher latency than the fallback source \
@@ -282,7 +272,7 @@ impl ObjectImpl for FallbackSrc {
                     0,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_int64(
+                glib::ParamSpecInt64::new(
                     "buffer-duration",
                     "Buffer Duration",
                     "Buffer duration when buffering streams (-1 default value)",
@@ -291,21 +281,21 @@ impl ObjectImpl for FallbackSrc {
                     -1,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_boxed(
+                glib::ParamSpecBoxed::new(
                     "statistics",
                     "Statistics",
                     "Various statistics",
                     gst::Structure::static_type(),
                     glib::ParamFlags::READABLE,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "manual-unblock",
                     "Manual unblock",
                     "When enabled, the application must call the unblock signal, except for live streams",
                     false,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "immediate-fallback",
                     "Immediate fallback",
                     "Forward the fallback streams immediately at startup, when the primary streams are slow to start up and immediate output is required",
@@ -663,6 +653,8 @@ impl ObjectImpl for FallbackSrc {
     }
 }
 
+impl GstObjectImpl for FallbackSrc {}
+
 impl ElementImpl for FallbackSrc {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -824,19 +816,11 @@ impl FallbackSrc {
                 let source = gst::ElementFactory::make("uridecodebin3", Some("uridecodebin"))
                     .expect("No uridecodebin3 found");
 
-                let uri = element
-                    .emit_by_name("update-uri", &[uri])
-                    .expect("Failed to emit update-uri signal")
-                    .expect("No value returned");
-                let uri = uri
-                    .get::<&str>()
-                    .expect("Wrong type returned from update-uri signal");
+                let uri = element.emit_by_name::<glib::GString>("update-uri", &[uri]);
 
-                source.set_property("uri", &uri).unwrap();
-                source.set_property("use-buffering", &true).unwrap();
-                source
-                    .set_property("buffer-duration", &buffer_duration)
-                    .unwrap();
+                source.set_property("uri", uri);
+                source.set_property("use-buffering", true);
+                source.set_property("buffer-duration", buffer_duration);
 
                 source
             }
@@ -845,7 +829,7 @@ impl FallbackSrc {
 
         // Handle any async state changes internally, they don't affect the pipeline because we
         // convert everything to a live stream
-        source.set_property("async-handling", &true).unwrap();
+        source.set_property("async-handling", true);
         // Don't let the bin handle state changes of the source. We want to do it manually to catch
         // possible errors and retry, without causing the whole bin state change to fail
         source.set_locked_state(true);
@@ -894,7 +878,7 @@ impl FallbackSrc {
         input.add_many(&[&audiotestsrc]).unwrap();
 
         audiotestsrc.set_property_from_str("wave", "silence");
-        audiotestsrc.set_property("is-live", &true).unwrap();
+        audiotestsrc.set_property("is-live", true);
 
         let srcpad = audiotestsrc.static_pad("src").unwrap();
         input
@@ -928,20 +912,18 @@ impl FallbackSrc {
         let clocksync = gst::ElementFactory::make("clocksync", None)
             .or_else(|_| -> Result<_, glib::BoolError> {
                 let identity = gst::ElementFactory::make("identity", None)?;
-                identity.set_property("sync", &true).unwrap();
+                identity.set_property("sync", true);
                 Ok(identity)
             })
             .expect("No clocksync or identity found");
 
         // Workaround for issues caused by https://gitlab.freedesktop.org/gstreamer/gst-plugins-base/-/issues/800
         let clocksync_queue = gst::ElementFactory::make("queue", None).expect("No queue found");
-        clocksync_queue
-            .set_properties(&[
-                ("max-size-buffers", &0u32),
-                ("max-size-bytes", &0u32),
-                ("max-size-time", &gst::ClockTime::SECOND),
-            ])
-            .unwrap();
+        clocksync_queue.set_properties(&[
+            ("max-size-buffers", &0u32),
+            ("max-size-bytes", &0u32),
+            ("max-size-time", &gst::ClockTime::SECOND),
+        ]);
 
         element
             .add_many(&[&fallback_input, &switch, &clocksync_queue, &clocksync])
@@ -957,13 +939,9 @@ impl FallbackSrc {
             let src = FallbackSrc::from_instance(&element);
             src.handle_switch_active_pad_change(&element);
         });
-        switch.set_property("timeout", &timeout.nseconds()).unwrap();
-        switch
-            .set_property("min-upstream-latency", &min_latency.nseconds())
-            .unwrap();
-        switch
-            .set_property("immediate-fallback", &immediate_fallback)
-            .unwrap();
+        switch.set_property("timeout", timeout.nseconds());
+        switch.set_property("min-upstream-latency", min_latency.nseconds());
+        switch.set_property("immediate-fallback", immediate_fallback);
 
         gst::Element::link_pads(&fallback_input, Some("src"), &switch, Some("fallback_sink"))
             .unwrap();
@@ -1265,11 +1243,8 @@ impl FallbackSrc {
 
         let mut is_image = false;
 
-        if let Some(stream_event) = pad.sticky_event(gst::EventType::StreamStart, 0) {
-            let stream = match stream_event.view() {
-                gst::EventView::StreamStart(ref ev) => ev.stream(),
-                _ => unreachable!(),
-            };
+        if let Some(ev) = pad.sticky_event::<gst::event::StreamStart<_>>(0) {
+            let stream = ev.stream();
 
             if let Some(stream) = stream {
                 if let Some(caps) = stream.caps() {
@@ -1341,7 +1316,7 @@ impl FallbackSrc {
 
             gst_debug!(CAT, "image stream, inserting imagefreeze");
             element.add(&imagefreeze).unwrap();
-            imagefreeze.set_property("is-live", &true).unwrap();
+            imagefreeze.set_property("is-live", true);
             if imagefreeze.sync_state_with_parent().is_err() {
                 gst_error!(CAT, obj: element, "imagefreeze failed to change state",);
                 return Err(gst::error_msg!(
@@ -1572,19 +1547,15 @@ impl FallbackSrc {
             None => return Ok(()),
         };
 
-        let ev = match pad.sticky_event(gst::EventType::Segment, 0) {
-            Some(ev) => ev,
+        let segment = match pad.sticky_event::<gst::event::Segment<_>>(0) {
+            Some(ev) => ev.segment().clone(),
             None => {
                 gst_warning!(CAT, obj: element, "Have no segment event yet");
                 return Ok(());
             }
         };
 
-        let segment = match ev.view() {
-            gst::EventView::Segment(s) => s.segment(),
-            _ => unreachable!(),
-        };
-        let segment = segment.downcast_ref::<gst::ClockTime>().ok_or_else(|| {
+        let segment = segment.downcast::<gst::ClockTime>().map_err(|_| {
             gst_error!(CAT, obj: element, "Have no time segment");
             gst::error_msg!(gst::CoreError::Clock, ["Have no time segment"])
         })?;
@@ -1593,9 +1564,9 @@ impl FallbackSrc {
         let running_time = if let Some((_, start)) =
             pts.zip(segment.start()).filter(|(pts, start)| pts < start)
         {
-            Some(start)
+            segment.to_running_time(start)
         } else if let Some((_, stop)) = pts.zip(segment.stop()).filter(|(pts, stop)| pts >= stop) {
-            Some(stop)
+            segment.to_running_time(stop)
         } else {
             segment.to_running_time(pts)
         };
@@ -2018,10 +1989,7 @@ impl FallbackSrc {
 
                 let prev_fallback_uri = video_stream
                     .fallback_input
-                    .property("uri")
-                    .unwrap()
-                    .get::<Option<String>>()
-                    .unwrap();
+                    .property::<Option<String>>("uri");
 
                 // This means previously videotestsrc was configured
                 // Something went wrong and there is no other way than to error out
@@ -2387,13 +2355,7 @@ impl FallbackSrc {
                 && state
                     .audio_stream
                     .as_ref()
-                    .and_then(|s| {
-                        s.switch
-                            .property("active-pad")
-                            .unwrap()
-                            .get::<Option<gst::Pad>>()
-                            .unwrap()
-                    })
+                    .and_then(|s| s.switch.property::<Option<gst::Pad>>("active-pad"))
                     .map(|p| p.name() == "fallback_sink")
                     .unwrap_or(true))
             || (have_video
@@ -2401,13 +2363,7 @@ impl FallbackSrc {
                 && state
                     .video_stream
                     .as_ref()
-                    .and_then(|s| {
-                        s.switch
-                            .property("active-pad")
-                            .unwrap()
-                            .get::<Option<gst::Pad>>()
-                            .unwrap()
-                    })
+                    .and_then(|s| s.switch.property::<Option<gst::Pad>>("active-pad"))
                     .map(|p| p.name() == "fallback_sink")
                     .unwrap_or(true))
     }
