@@ -1,19 +1,10 @@
 // Copyright (C) 2019 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::gst_debug;
 use gst::prelude::*;
@@ -371,30 +362,27 @@ fn setup_pipeline(with_live_fallback: Option<bool>) -> Pipeline {
         .unwrap()
         .downcast::<gst_app::AppSrc>()
         .unwrap();
-    src.set_property("is-live", &true).unwrap();
-    src.set_property("format", &gst::Format::Time).unwrap();
-    src.set_property("min-latency", &(10i64)).unwrap();
+    src.set_property("is-live", true);
+    src.set_property("format", gst::Format::Time);
+    src.set_property("min-latency", 10i64);
     src.set_property(
         "caps",
         &gst::Caps::builder("video/x-raw")
-            .field("format", &"ARGB")
-            .field("width", &320)
-            .field("height", &240)
-            .field("framerate", &gst::Fraction::new(1, 1))
+            .field("format", "ARGB")
+            .field("width", 320)
+            .field("height", 240)
+            .field("framerate", gst::Fraction::new(1, 1))
             .build(),
-    )
-    .unwrap();
+    );
 
     let switch = gst::ElementFactory::make("fallbackswitch", Some("switch")).unwrap();
-    switch
-        .set_property("timeout", &(3 * gst::ClockTime::SECOND))
-        .unwrap();
+    switch.set_property("timeout", 3 * gst::ClockTime::SECOND);
 
     let sink = gst::ElementFactory::make("appsink", Some("sink"))
         .unwrap()
         .downcast::<gst_app::AppSink>()
         .unwrap();
-    sink.set_property("sync", &false).unwrap();
+    sink.set_property("sync", false);
 
     let queue = gst::ElementFactory::make("queue", None).unwrap();
 
@@ -410,22 +398,18 @@ fn setup_pipeline(with_live_fallback: Option<bool>) -> Pipeline {
             .unwrap()
             .downcast::<gst_app::AppSrc>()
             .unwrap();
-        fallback_src.set_property("is-live", &live).unwrap();
-        fallback_src
-            .set_property("format", &gst::Format::Time)
-            .unwrap();
-        fallback_src.set_property("min-latency", &(10i64)).unwrap();
-        fallback_src
-            .set_property(
-                "caps",
-                &gst::Caps::builder("video/x-raw")
-                    .field("format", &"ARGB")
-                    .field("width", &160)
-                    .field("height", &120)
-                    .field("framerate", &gst::Fraction::new(1, 1))
-                    .build(),
-            )
-            .unwrap();
+        fallback_src.set_property("is-live", live);
+        fallback_src.set_property("format", gst::Format::Time);
+        fallback_src.set_property("min-latency", 10i64);
+        fallback_src.set_property(
+            "caps",
+            &gst::Caps::builder("video/x-raw")
+                .field("format", "ARGB")
+                .field("width", 160)
+                .field("height", 120)
+                .field("framerate", gst::Fraction::new(1, 1))
+                .build(),
+        );
 
         pipeline.add(&fallback_src).unwrap();
 
